/*
 *
 * CacheObject.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: CacheObject.h,v $
 * Revision 1.7  2007/07/14 07:50:21  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.6  2007/06/18 15:30:01  joegenbaclor
 * Implemented Internal DNS Mapping
 * Fixed bug in Cache Manager
 *
 * Revision 1.5  2007/06/16 12:30:35  joegenbaclor
 * Changed CacheManager to return a smart pointer instead of a direct pointer to cache object to avoid segmentation faults when an object expires in cache and another thread has pending ownership of the pointer
 *
 * Renamed CacheManager::GetAt() to CacheManager::CreatePointer() to give a hint to users that the pointer needs to be deleted after being used
 *
 * Revision 1.4  2006/11/30 08:19:35  joegenbaclor
 * Relicensed opensipstack to MPL/GPL/LGPL triple license scheme
 *
 * Revision 1.3  2006/03/14 03:13:21  joegenbaclor
 * Initial header upload for BETA Release Candidate
 *
 *
 */

#ifndef CACHE_H
#define CACHE_H

#include <ptlib.h>
#include "OString.h"

using namespace Tools;

namespace Cache
{
  class CacheObject : public PObject
  {
    PCLASSINFO( CacheObject, PObject );

  public:

    CacheObject(
      const OString & objectIdentifier,
      PObject * cacheObject,
      int lifeSpanInSeconds = INT_MAX
    );

    virtual ~CacheObject();

    PINLINE int GetLifeSpan()const{ return m_LifeSpanInterval; };
    PINLINE int DecreaseLifeSpan(){ return --m_LifeSpanInterval; };
    PINLINE void SetLifeSpan( int life ){ m_LifeSpanInterval = life; };
    PINLINE const OString & GetObjectIdentifier()const{ return m_ObjectIdentifier; };
    PINLINE PObject * GetObject(){ return m_Object; };
    PINLINE void AllowExpire(){ m_CanExpire = TRUE; };
    PINLINE void DisallowExpire(){ m_CanExpire = FALSE; }
    PINLINE BOOL IsExpireAllowed()const{ return m_CanExpire; };
    PDICTIONARY( SortedCollection, PCaselessString, CacheObject );
    PLIST( Collection, CacheObject );
  protected:

    OString m_ObjectIdentifier;
    PObject * m_Object;
    int m_LifeSpanInterval;
    BOOL m_CanExpire;
  };

  class CachePointer : public PObject
  {
    PCLASSINFO( CachePointer, PObject );
  protected:
    CacheObject * m_Obj;
  public:
    CachePointer( CacheObject * obj )
    {
      if( obj != NULL )
      {
        m_Obj = obj;
        obj->DisallowExpire();
      }
    }

    ~CachePointer()
    {
      if( m_Obj != NULL)
        m_Obj->AllowExpire();
    }

    PINLINE CacheObject * operator->()
    {
      return m_Obj;
    }

    PINLINE PObject * GetObject()
    { 
      if( m_Obj == NULL )
        return NULL;

      return m_Obj->GetObject(); 
    }

    PINLINE CachePointer & operator=( CacheObject * obj )
    {
      m_Obj = obj;
      obj->DisallowExpire();
      return *this;
    }

  private:
    PINLINE CachePointer & operator=( CachePointer & /*obj*/ )
    {
      return *this;
    }
  };
};

#endif



