/*
 *
 * SoftPhoneInterface.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: SoftPhoneInterface.h,v $
 * Revision 1.25  2007/07/14 07:50:28  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.24  2007/05/31 03:38:43  joegenbaclor
 * Reoved extra qualifier from Ilians last check in
 *
 * Revision 1.23  2007/05/30 07:39:27  ijpinzon
 * 1. Exposed setting of silence detection mode
 * 2. Exposed setting of audio jitter delay
 *
 * Revision 1.22  2007/05/22 05:20:37  ijpinzon
 * Exposed setting of encryption mode and encryption key in SetEnableEncryption
 *
 * Revision 1.21  2007/05/08 07:06:38  ijpinzon
 * 1. Implemented ring device
 * 2. Extended tone playback functions to support playing on a custom sound channel
 *
 * Revision 1.20  2007/04/23 11:37:45  ijpinzon
 * 1.  Modified SetVolume to support the new volume control in SoftphoneManager
 * 2.  Implemented GetVolume for SoftphoneInterface
 * 3.  Added pure virtual function Event_VolumeChanged
 *
 * Revision 1.19  2007/04/13 09:41:55  joegenbaclor
 * Added Allow header in INVITE and REGISTER
 *
 * Revision 1.18  2007/04/10 06:13:07  joegenbaclor
 * Added new Encryption Mode
 *
 * Revision 1.17  2007/02/19 11:31:02  joegenbaclor
 * more privacy work
 *
 * Revision 1.16  2007/02/10 03:03:09  joegenbaclor
 * Added SetInterfaceAddress() function to SoftPhoneManager
 *
 * Revision 1.15  2007/02/02 06:55:47  joegenbaclor
 * Added OnTerminated Event and syncrhonous Unregister
 *
 * Revision 1.14  2007/01/26 01:45:57  joegenbaclor
 * Changed name of GetAccountBalance to RPCGetAccountBalance
 *
 * Revision 1.13  2007/01/01 05:12:52  joegenbaclor
 * Introduced termiantion flag to avoid crashes on exit.
 * TO DO: Maybe the best thing to do here is to have a WaitForTermination()
 * function exposed for the SIPStack
 *
 * Revision 1.12  2006/12/19 23:38:31  joegenbaclor
 * Fixed bug in softphone proxy authentication where passwords <= 3 in length may not be properly handled
 *
 * Revision 1.11  2006/12/14 09:49:56  joegenbaclor
 * Tweaked cleanup routines
 *
 * Revision 1.10  2006/12/12 23:53:32  joegenbaclor
 * More work on DTMF encoding
 *
 * Revision 1.9  2006/12/12 05:26:08  joegenbaclor
 * Modified DTMF player to use PDTMFEncoder
 *
 * Revision 1.8  2006/12/08 09:56:13  joegenbaclor
 * Added Hold functions
 *
 * Revision 1.7  2006/12/07 09:12:21  joegenbaclor
 * 1.  Changed registration expire value from milliseconds to seconds
 *
 * Revision 1.6  2006/12/06 10:17:11  joegenbaclor
 * More SF enhancement
 *
 * Revision 1.5  2006/12/06 06:40:54  joegenbaclor
 * STUN related fixes
 *
 * Revision 1.4  2006/12/06 01:59:22  joegenbaclor
 * More Softphone support
 *
 * Revision 1.3  2006/12/04 08:55:32  joegenbaclor
 * Added log hook to transport
 *
 * Revision 1.2  2006/12/04 07:32:49  joegenbaclor
 * Added RFC3680 methods to Opal classes
 *
 * Revision 1.1  2006/12/02 01:12:13  joegenbaclor
 * Initial upload of SoftPhoneInterface classes
 *
 *
 */


#ifndef SOFTPHONEINTERFACE_H
#define SOFTPHONEINTERFACE_H

#include <ptlib.h>
#include "Encryption.h"
#include "SIPNATMethods.h"

namespace SF
{
  class SoftPhoneManager;

  class SoftPhoneInterface
  {
  public:
    SoftPhoneInterface();

    virtual ~SoftPhoneInterface();

    virtual void CleanUp();

    void Terminate();

    /// pure virtuals
    virtual void  Event_PCSoundSystemInitialized( 
      const OString & recordDevice,
      const OString & soundDevice
    ) = 0;

    virtual void  Event_SIPInitialized(
      const OString & eventInfo = OString::Empty()
    ) = 0;
    
    virtual void  Event_SIPInitializationError( 
      int errorCode, 
      const OString & errorInfo 
    ) = 0;

    virtual void  Event_OutgoingCallRejected( 
      int errorCode, 
      const OString & errorInfo 
    ) = 0;

    virtual void  Event_IncomingCallConnected( 
      const OString & eventInfo  = OString::Empty()
    ) = 0;
    
    virtual void  Event_CallDisconnected(
      const OString & eventInfo  = OString::Empty()
    ) = 0;
    
    virtual void  Event_LoginError( 
      int errorCode, 
      const OString & errorInfo 
    ) = 0;

    virtual void  Event_LoginSuccessful(
      const OString & eventInfo  = OString::Empty()
    ) = 0;
    
    virtual void  Event_LogoutSuccessful(
      const OString & eventInfo  = OString::Empty()
    ) = 0;

    virtual void  Event_OutgoingCallConnected( 
      const OString & eventInfo  = OString::Empty()
    ) = 0;
    
    virtual void  Event_IncomingCall( 
      const OString & eventInfo  = OString::Empty()
    ) = 0;

    virtual void  Event_OutgoingCallRinging( 
      const OString & eventInfo  = OString::Empty()
    ) = 0;

    virtual void  Event_IncomingInstantMessage( 
      const OString & senderInfo,
      const OString & contentType,
      const OString & messageBody
    ) = 0;

    virtual void Event_RegEventStateOnline(
      const OString & user,
      const OString & domain
    ) = 0;

    virtual void Event_RegEventStateOffLine(
      const OString & user,
      const OString & domain
    ) = 0;

    virtual void Event_WritePacketLog(
      const OString & log
    ) = 0;

    virtual void Event_ReadPacketLog(
      const OString & log
    ) = 0;

    virtual void Event_Terminated() = 0;

    virtual void Event_VolumeChanged(
      const unsigned speakerVolume,
      const unsigned micVolume
    )  = 0;

    /// Initialize SIP and PC Sound System
    BOOL Initialize();

    /// Perform Registration
    BOOL DoLogin();

    /// Perform unregistration
    BOOL DoLogout();


    /// add a contact to the address book
    BOOL AddContact(
	    const OString & displayName, 
	    const OString & sipURI, 
          BOOL queryPresence
    );
      
    /// returns a contact by index
    BOOL GetContact( 
      PINDEX index, 
      OString & accountName, 
      OString & sipURI, 
      BOOL & queryPresence
    )const;

    BOOL GetContactByName(
	    const OString & accountName, 
          OString & contactAddress, 
          BOOL & queryPresence
    )const;

    /// returns the size of the address book
    PINDEX GetContactSize()const;

    /// deletes a contact by name
    BOOL DeleteContact( 
	    const OString & contact
    );

    /// deletes a contact using the index number 
    BOOL DeleteContactByIndex(
	    PINDEX index
    );
      
    /// returns the name of this account.
    /// this is also the displayname sent
    OString GetAccountName()const;

    /// sets the name of this account.
    /// this is also the displayname sent
    void SetAccountName(
	    const OString & accountName
    );

    /// returns the SIP URI for the softphone
    /// this is also the SIP URI the softphone
    /// sends for the TO header
    OString GetAccountAddress()const;

    /// sets the SIP URI for the softphone
    /// this is also the SIP URI the softphone
    /// sends for the TO header
    void SetAccountAddress(
	    const OString & accountAddress
    );

    /// returns the address of the Registrar
    /// this is the address where registrations 
    /// will be sent
    OString GetSIPServerAddress()const;

    /// returns the address of the Registrar
    /// this is the address where registrations 
    /// will be sent
    void SetSIPServerAddress(
      const OString & severAddress
    );


    /// returns the address for the outbound proxy
    OString GetOutboundProxy()const;

    /// returns the address for the outbound proxy
    void SetOutboundProxy(
	    const OString & proxyAddress
    );

    /// returns the user name sent by the softphone
    /// when sending authentication and authorization
    /// responses
    OString GetAuthenticationUser()const;

    /// sets the user name sent by the softphone
    /// when sending authentication and authorization
    /// responses
    void SetAuthenticationUser(
	    const OString & authUser
    );

    /// returns the password sent by the softphone
    /// when sending authentication and authorization
    /// responses
    OString GetAuthenticationPassword()const;

    /// sets the password sent by the softphone
    /// when sending authentication and authorization
    /// responses
    void SetAuthenticationPassword(
	    const OString & authPassword
    );

    /// returns the user name sent by the softphone
    /// when sending authentication and authorization
    /// responses
    OString GetProxyAuthenticationUser()const;

    /// sets the user name sent by the softphone
    /// when sending authentication and authorization
    /// responses
    void SetProxyAuthenticationUser(
	    const OString & authUser
    );

    /// returns the password sent by the softphone
    /// when sending authentication and authorization
    /// responses
    OString GetProxyAuthenticationPassword()const;

    /// sets the password sent by the softphone
    /// when sending authentication and authorization
    /// responses
    void SetProxyAuthenticationPassword(
	    const OString & authPassword
    );


    /// adds a new account to the account database.
    BOOL AddAccount(
	    const OString & accountName,
      const OString & accountAddress,
	    const OString & authenticationUser, 
	    const OString & authenticationPassword, 
      const OString & proxyAuthenticationUser, 
	    const OString & proxyAuthenticationPassword, 
	    const OString & serverAddress,
      const OString & outboundProxy, 
	    BOOL isDefaultAccount, 
	    BOOL isAutoLogin
    );


    /// returns the size of the account database
    PINDEX GetAccountSize()const;

    /// returns an account using its index in the 
    /// account database
    BOOL GetAccountByIndex(
	    PINDEX index,
	    OString & key, 
	    OString & accountName, 
	    OString & accountAddress, 
	    OString & serverAddress, 
	    OString & outboundProxy, 
	    OString & authenticationUser, 
	    OString & authenticationPassword, 
      OString & proxyAuthenticationUser, 
	    OString & proxyAuthenticationPassword, 
	    BOOL & isDefaultAccount, 
	    BOOL & isAutoLogin
    )const;

    /// returns an account using its key in the 
    /// account database which might have been known
    /// by the application from previous call to GetAccountByIndex
    BOOL GetAccountByKey(
	    const OString & key, 
	    OString & accountName, 
	    OString & accountAddress, 
	    OString & serverAddress, 
	    OString & outboundProxy, 
	    OString & authenticationUser, 
	    OString & authenticationPassword, 
      OString & proxyAuthenticationUser, 
	    OString & proxyAuthenticationPassword, 
	    BOOL & isDefaultAccount, 
	    BOOL & isAutoLogin
    )const;

    /// deletes an account using its key in the 
    /// account database which might have been known
    /// by the application from previous call to GetAccountByIndex
    BOOL DeleteAccountByKey(
	    const OString & key
    );

    /// deletes an account using its index in the 
    /// account database
    BOOL DeleteAccountByIndex(
	    PINDEX index
    );


    /// returns the detected record devices in a string array
    /// (microphones)
    PStringList PopAvailableRecordDevices()const;

    /// returns the detected play devices in a string array
    /// (speakers)
    PStringList PopAvailablePlayDevices()const;

    /// returns the detected video input devices in a string array
    /// (video camera)
    PStringList PopAvailableVideoInputDevices()const;

    /// returns the available codecs in a string array
    PStringArray PopAvailableAudioCodecs(void);

    /// returns the entries in the call history database
    PStringToString PopCallHistory( PINDEX count );

    /// returns the name of the active microphone
    /// device 
    OString GetDefaultMicrophone()const;

    /// sets the name of the active microphone
    /// device 
    void SetDefaultMicrophone(
	    const OString & deviceName
    );

    /// returns the name of the active speaker device
    OString GetDefaultSpeaker()const;

    void SetDefaultSpeaker(
	    const OString & deviceName
    );

    /// Returns the name of the active ring device
    OString GetDefaultRingDevice()const;

    void SetDefaultRingDevice(
	    const OString & deviceName
    );

    /// set the default speaker/microphone volume
    void SetVolume( 
      unsigned volume,
      const BOOL isPlayerDevice = TRUE
    );

    /// gets the default speaker/microphone volume
    void GetVolume( 
      unsigned & volume,
      const BOOL isPlayerDevice = TRUE
    );

    /// returns the codec based on its slot number.
    /// slot == 1 means highest priority in the codec list 
    OString GetAudioCodec( 
	    PINDEX slot
    )const;

    /// sets the codec based on its slot number.
    /// slot == 1 means highest priority in the codec list. 
    void SetAudioCodec(
	    PINDEX slot,
	    const OString & codecName
    );

    /// Plays an audible dtmf tone on the softphone's default audio 
    /// player device. Take note that this will not send a DTMF tone 
    /// over the wire. Use SendRFC2833Tone instead.
    void PlayDTMF(
      const int tone
    );

    /// Plays an audible dtmf tone on a specified sound channel.
    /// Take not that this will not send a DTMF tone over the wire.
    /// Use SendRFC2833Tone instead.
    void PlayDTMF(
      const int tone,
      PSoundChannel & playChannel
    );

    /// Plays an audible dtmf tone on the softphone's default audio 
    /// player device. Take note that this will not send a DTMF tone 
    /// over the wire. Use SendRFC2833Tone instead.
    void PlayDTMF(
      const OString & tone
    );

    /// Plays an audible dtmf tone on a specified sound channel.
    /// Take not that this will not send a DTMF tone over the wire.
    /// Use SendRFC2833Tone instead.
    void PlayDTMF(
      const OString & tone,
      PSoundChannel & playChannel
    );

    /// sets the STUN server to be used for NAT bindings
    NAT::SIPSTUNClient::NatTypes SetSTUNServer(
      const OString & server,
      WORD udpPortBase = 10000,
      WORD udpPortMax = 20000,
      WORD rtpPortBase = 30000,
      WORD rtpPortMax = 40000
    );

    ///  returns the STUN server address to be used for NAT bindings
    OString GetSTUNServerAddress()const;

    /// makes an outbound call to this address.
    /// address should contain a valid SIP URI
    void MakeCall( 
	    const OString & address
    );

    /// hangup the current call
    void HangupCall();

    /// answers the current call
    void AnswerCall();

    /// Sends a plain text MESSAGE to remoteAddress.
    /// remoteAddress should be a valid SIP URI.
    void SendInstantMessage(
      const OString & remoteAddress, 
      const OString & message
    );

    /// Returns the call history inside a string array.
    /// count contains the number of calls to be returned
    PStringToString GetCallHistory( 
      PINDEX count 
    )const;


    /// Sends an RFC 2833 DTMF tone over the wire
    void SendRFC2833Tone(
      const OString & tone, 
      int duration
    );

    /// Sends a SIP INFO DTMF tone over the wire
    void SendINFOTone(
      const OString & tone, 
      int duration
    );

    /// sends a DTMF tone.  Default behavior uses RFC2833
    void SendDTMF(
      const OString & tone, 
      int duration = 180
    );

    /// sets RFC 2833 as the default method for sending DTMF
    void SetEnableRFC2833( BOOL enable = TRUE );

    /// check if RFC 2833 is the default method for sending DTMF
    BOOL IsRFC2833Enabled()const;


    /// enable hashing of RTP and SIP over the wire
    /// take note that this is a none standard mechanism.
    /// this will not work for servers other than opensbc
    void SetEnableEncryption( 
      BOOL enable = TRUE, 
      Encryption::Engine::Type type = Encryption::Engine::Type_XOR, 
      const OString & key = "GS"
    );

    /// check if xor hashing for RTP and SIP is enabled
    BOOL IsEncryptionEnabled()const;

    /// enable Privacy (RFC3325)
    void SetEnablePrivacy( BOOL enable = TRUE );

    /// check if Privacy (RFC3325) is enabled
    BOOL IsPrivacyEnabled()const;

    /// Set the maximum registration keep-alive interval
    void SetRegistrarTTL( 
      const PTimeInterval & ttl 
    );

    /// Returns the maximum registration keep alive interval
    PTimeInterval GetRegistrarTTL()const;


    /// Returns the interface address used by the softphone
    OString GetInterfaceAddress()const;

    /// Sets the interface address the softphone will bind to
    void SetInterfaceAddress( 
      const OString & iface 
    );

    /// RFC 3680 support
    void WatchRegistrationState(
      const OString & stateAgent,
      const OString & resourceURI,
      const PTimeInterval & expires = PTimeInterval( 0, 90 ),
      const OString & authUser = OString::Empty(),
      const OString & authPassword = OString::Empty()
      
    );

    void StopWatchingRegistrationState(
      const OString & resourceURI
    );

	  /// Gets Acount Balance Via XML RPC
	  double RPCGetAccountBalance(
      const OString & server,
      const OString & account
    );

    /// Indicates whether the softphone can make outbound calls
    /// when not registered
    void SetRequireRegistration(
      BOOL require
    );

    BOOL WillRequireRegistration()const;

    /// returns the current signaling state of the softphone
    int GetState()const;

    /// Disables silence detection. This effectively stops comfort
    /// noise generation (except for pure silent packets).
    void DisableSilenceDetection();

    /// Enables adaptive silence detection. Threshold level 
    /// adapts to the signal dynamically every adaptivePeriod 
    /// millisecs.
    void EnableAdaptiveSilenceDetection(
      unsigned adaptivePeriod = 4800,
      unsigned signalDeadband = 80,
      unsigned silenceDeadband = 3200
    );

    /// Enables fixed silence detection. Any sound level below the 
    /// threshold is treated as silence.
    void EnableFixedSilenceDetection(
      unsigned threshold = 0,
      unsigned signalDeadband = 80,
      unsigned silenceDeadband = 3200
    );

    /// Sets audio jitter delay
    void SetAudioJitterDelay(
      unsigned minDelay, 
      unsigned maxDelay
    );

    PConfig * GetMainConfig();
    PConfig * GetContacts();
    PConfig * GetAccounts();
    PConfig * GetCallHistory();
    SoftPhoneManager * GetPhoneManager();

    NAT::SIPSTUNClient::NatTypes m_NATType;
    
    PDECLARE_NOTIFIER( PObject, SoftPhoneInterface, _OnWritePacketLog );
    PDECLARE_NOTIFIER( PObject, SoftPhoneInterface, _OnReadPacketLog );

    /// Plays an audible busy sound.
    void PlayBusyTone();
    void PlayBusyTone( const OString & device );
    void StopBusyTone();
    BOOL m_PlayBusyTone;
    PDECLARE_NOTIFIER( PThread, SoftPhoneInterface, PlayBusyTone );

    /// Plays an audible dialtone sound.
    void PlayDialTone();
    void PlayDialTone( const OString & device );
    void StopDialTone();
    BOOL m_PlayDialTone;
    PDECLARE_NOTIFIER( PThread, SoftPhoneInterface, PlayDialTone );
    
    /// Plays an audible ringback sound.
    void PlayRingBackTone();
    void PlayRingBackTone( const OString & device );
    void StopRingBackTone();
    BOOL m_PlayRingBackTone;
    PDECLARE_NOTIFIER( PThread, SoftPhoneInterface, PlayRingBackTone );
  };
};
#endif

